﻿using System;
using System.Threading;

namespace producerConsumerState
{
    public class UnboundedBuffer<T> : AbstractBuffer<T>
    {

        public UnboundedBuffer(String name)
            : base(name)
        {
        }

        public override T take()
        {
            if (ended) { throw new InvalidOperationException("Trying to take after ending"); }
            Monitor.Enter(queue);
            try
            {

                while (IsEmpty())
                {
                    Monitor.Wait(queue);
                }
                T element = queue.Dequeue();
                if (ended && queue.Count == 0)
                {
                    ended = true;
                }
                return element;
            }
            finally
            {
                Monitor.Exit(queue);
            }
        }

        public override void Put(T element)
        {
            if (ending) { throw new InvalidOperationException("Trying to put after putting the STOP_VALUE"); }
            Monitor.Enter(queue);
            try
            {
                queue.Enqueue(element);
                if (Count == 1)
                {
                    Monitor.PulseAll(queue);
                }
            }
            finally
            {
                Monitor.Exit(queue);
            }
        }
    }
}
