﻿using System;

namespace producerConsumer
{
    /// <summary>
    /// Duplicator is worker that takes from one buffer, and copies the element to TWO outgoing buffers
    /// </summary>
    public class Duplicator
    {
        private readonly IBuffer _incoming;
        private readonly IBuffer _outgoing1, _outgoing2;

        /// <summary>
        /// Constructs an object of this class.
        /// </summary>
        /// <param name="incoming">The buffer to take from</param>
        /// <param name="outgoing1">The first buffer to add to</param>
        /// <param name="outgoing2">The second buffer to add to</param>
        public Duplicator(IBuffer incoming, IBuffer outgoing1, IBuffer outgoing2)
        {
            _incoming = incoming;
            _outgoing1 = outgoing1;
            _outgoing2 = outgoing2;
        }

        /// <summary>
        /// Runs this object.<br />
        /// Stops when it takes Producer.LastItem
        /// </summary>
        public void Run()
        {
            while (true)
            {
                int element = _incoming.Take();
                Console.WriteLine("Duplicator consumed {0}", element);
                if (element == Producer.LastElement)
                {
                    break;
                }
                _outgoing1.Add(element);
                _outgoing2.Add(element);
            }
            _outgoing1.Add(Producer.LastElement);
            _outgoing2.Add(Producer.LastElement);
        }
    }
}
